@@ -2,7 +2,6 @@ import numbers as _numbers
 from contextvars import ContextVar
 from fractions import Fraction as _Fraction
 from typing import (Sequence,
-                    Tuple,
                     Type)
 
 from reprit.base import generate_repr
@@ -18,27 +17,24 @@ from .core import (angular as _angular,
                    linear as _linear,
                    vector as _vector)
 from .core.hints import QuaternaryPointFunction as _QuaternaryPointFunction
-from .hints import Point
 
 _QuaternaryFunction = _QuaternaryPointFunction[_hints.Coordinate]
 Kind = _enums.Kind
 Orientation = _enums.Orientation
-SegmentsRelationship = _enums.SegmentsRelationship
+Relation = _enums.Relation
 
 
 class Context:
     __slots__ = ('_box_cls', '_centroidal', '_contour_cls', '_coordinate_cls',
-                 '_incircle', '_inverse', '_multicontour_cls',
-                 '_multipoint_cls', '_multipolygon_cls', '_multisegment_cls',
-                 '_point_cls', '_polygon_cls', '_segment_cls', '_vector')
+                 '_incircle', '_inverse', '_linear', '_multipoint_cls',
+                 '_multipolygon_cls', '_multisegment_cls', '_point_cls',
+                 '_polygon_cls', '_segment_cls', '_vector')
 
     def __init__(self,
                  *,
                  box_cls: Type[_hints.Box] = _geometries.Box,
                  contour_cls: Type[_hints.Contour] = _geometries.Contour,
                  coordinate_cls: Type[_hints.Coordinate] = _numbers.Real,
-                 multicontour_cls: Type[_hints.Multicontour]
-                 = _geometries.Multicontour,
                  multipoint_cls: Type[_hints.Multipoint]
                  = _geometries.Multipoint,
                  multipolygon_cls: Type[_hints.Multipolygon]
@@ -52,7 +48,6 @@ class Context:
         self._box_cls = box_cls
         self._contour_cls = contour_cls
         self._coordinate_cls = coordinate_cls
-        self._multicontour_cls = multicontour_cls
         self._multipoint_cls = multipoint_cls
         self._multipolygon_cls = multipolygon_cls
         self._multisegment_cls = multisegment_cls
@@ -62,12 +57,12 @@ class Context:
         self._inverse = (1.
                          if issubclass(coordinate_cls, float)
                          else _Fraction(1)).__truediv__
-        self._centroidal, self._incircle, self._vector = (
+        self._centroidal, self._incircle, self._linear, self._vector = (
             (_centroidal.plain_context, _incircle.plain_context,
-             _vector.plain_context)
+             _linear.plain_context, _vector.plain_context)
             if issubclass(coordinate_cls, _numbers.Rational)
             else (_centroidal.robust_context, _incircle.robust_context,
-                  _vector.robust_context))
+                  _linear.robust_context, _vector.robust_context))
 
     __repr__ = generate_repr(__init__)
 
@@ -91,10 +86,6 @@ class Context:
     def dot_product(self) -> _QuaternaryFunction:
         return self._vector.dot_product
 
-    @property
-    def multicontour_cls(self) -> Type[_hints.Multicontour]:
-        return self._multicontour_cls
-
     @property
     def multipoint_cls(self) -> Type[_hints.Multipoint]:
         return self._multipoint_cls
@@ -137,56 +128,120 @@ class Context:
         return _angular.orientation(self.cross_product, vertex,
                                     first_ray_point, second_ray_point)
 
-    def contour_centroid(self, contour: _hints.Contour) -> _hints.Point:
+    def contour_centroid(self,
+                         vertices: Sequence[_hints.Point]) -> _hints.Point:
+        """
+        Constructs centroid of a contour given its vertices.
+
+        Time complexity:
+            ``O(len(vertices))``
+        Memory complexity:
+            ``O(1)``
+
+        >>> context = get_context()
+        >>> Point = context.point_cls
+        >>> context.contour_centroid([Point(0, 0), Point(2, 0), Point(2, 2),
+        ...                           Point(0, 2)]) == Point(1, 1)
+        True
+        """
         return self._centroidal.contour_centroid(self._inverse, self.point_cls,
-                                                 contour)
+                                                 vertices)
 
     def merged_box(self, first_box: _hints.Box, second_box: _hints.Box
                    ) -> _hints.Box:
         return _boxed.merge(self.box_cls, first_box, second_box)
 
     def multipoint_centroid(self,
-                            multipoint: _hints.Multipoint) -> _hints.Point:
-        return self._centroidal.multipoint_centroid(self.point_cls, multipoint)
+                            points: Sequence[_hints.Point]) -> _hints.Point:
+        """
+        Constructs centroid of a multipoint given its points.
 
-    def points_convex_hull(self, points: Sequence[Point]) -> Sequence[Point]:
+        Time complexity:
+            ``O(len(points))``
+        Memory complexity:
+            ``O(1)``
+
+        >>> context = get_context()
+        >>> Point = context.point_cls
+        >>> context.multipoint_centroid([Point(0, 0), Point(2, 0), Point(2, 2),
+        ...                              Point(0, 2)]) == Point(1, 1)
+        True
+        """
+        return self._centroidal.multipoint_centroid(self.point_cls, points)
+
+    def points_convex_hull(self,
+                           points: Sequence[_hints.Point]
+                           ) -> Sequence[_hints.Point]:
+        """
+        Constructs convex hull of points.
+
+        Time complexity:
+            ``O(points_count * log(points_count))``
+        Memory complexity:
+            ``O(points_count)``
+        where ``points_count = len(points)``.
+
+        >>> context = get_context()
+        >>> Point = context.point_cls
+        >>> (context.points_convex_hull([Point(0, 0), Point(2, 0), Point(2, 2),
+        ...                              Point(0, 2)])
+        ...  == [Point(0, 0), Point(0, 2), Point(2, 2), Point(2, 0)])
+        True
+        """
         return _discrete.to_convex_hull(self.angle_orientation, points)
 
     def segment_contains_point(self,
                                start: _hints.Point,
                                end: _hints.Point,
                                point: _hints.Point) -> bool:
-        return _linear.segment_contains_point(self.cross_product, start, end,
-                                              point)
+        """
+        Checks if a segment given by its endpoints contains given point.
+
+        Time complexity:
+            ``O(1)``
+        Memory complexity:
+            ``O(1)``
+
+        >>> context = get_context()
+        >>> Point = context.point_cls
+        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        ...                                Point(0, 0))
+        True
+        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        ...                                Point(0, 2))
+        False
+        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        ...                                Point(1, 0))
+        True
+        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        ...                                Point(1, 1))
+        False
+        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        ...                                Point(2, 0))
+        True
+        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        ...                                Point(3, 0))
+        False
+        """
+        return self._linear.containment_checker(self.cross_product, start, end,
+                                                point)
 
     def segments_intersection(self,
                               first_start: _hints.Point,
                               first_end: _hints.Point,
                               second_start: _hints.Point,
                               second_end: _hints.Point) -> _hints.Point:
-        return _linear.segments_intersection(
-                self.cross_product, self._inverse, self.point_cls, first_start,
-                first_end, second_start, second_end)
+        return self._linear.intersector(
+                self.cross_product, self.point_cls, first_start, first_end,
+                second_start, second_end)
 
-    def segments_intersections(self,
-                               first_start: _hints.Point,
-                               first_end: _hints.Point,
-                               second_start: _hints.Point,
-                               second_end: _hints.Point
-                               ) -> Tuple[_hints.Point, ...]:
-        return _linear.segments_intersections(
-                self.cross_product, self._inverse, self.point_cls, first_start,
-                first_end, second_start, second_end)
-
-    def segments_relationship(self,
-                              first_start: _hints.Point,
-                              first_end: _hints.Point,
-                              second_start: _hints.Point,
-                              second_end: _hints.Point
-                              ) -> SegmentsRelationship:
-        return _linear.segments_relationship(self.cross_product, first_start,
-                                             first_end, second_start,
-                                             second_end)
+    def segments_relation(self,
+                          first_start: _hints.Point,
+                          first_end: _hints.Point,
+                          second_start: _hints.Point,
+                          second_end: _hints.Point) -> Relation:
+        return self._linear.relater(self.cross_product, first_start, first_end,
+                                    second_start, second_end)
 
 
 _context = ContextVar('context',
